function [valid_particle, fitness] =  fitness(a,D,particle,vecDim)
% % % % 
% % % % Computes the fitness value of a particle, based on the cross
% % % % validation BER
% % % % 
% % % 
% % % % Input:  a           - A psms object
% % %           D           - training data 
% % %           particle    - a vector-particle, representing a model
% % %           vecDim      - Dimensionality of data
% % % 
% % % % Output: valid_particle  - the valid particle
% % %           fitness         - the fitness value of the valid particle
% % % 
% % %===============================================================
% % % Hugo Jair Escalante -- hugojair@ccc.inaoep.mx -- November 2007
% % %                        hugo.jair@gmail.com
% % %===============================================================
% % % 

% % % If you want to ommit some methods just set its respective values to
% % % zero for example
%         particle(1)=0;  %% do not considers s2n
%         particle(4)=0;  %% do not considers gs
%         particle(6)=0;  %% do not considers relief
%         particle(18)=0; %% do not considers pca
%         particle(10)=0; %% do not considers relief

% % % Validate the particle, according to upper and lower bounds on the
% % % parameters
        [valid_particle] = validate_particle(a,particle, vecDim);
        if (a.fms==-1)
            zero_fields=[1,4,6,10,12,14,16,18];
            valid_particle(:,zero_fields)=0;
            [my_model,base_model]=create_CLOP_model(a,valid_particle);
       else
% % %  Create the model using the validated particle       
        [my_model,base_model]=create_CLOP_model(a,valid_particle);
        end

        % Train the model
        %================
        debug=1; % Set to 1 to compute the training error in the process of training
        guessed_ber=[];
        cv_ber=[];
        chain_length=[];
        correct_bias=0;         % For correcting the BIAS
        if a.FoldNum > 0
            tic
            % Create a CV model
            cv_model    = cv(my_model, {['folds=' num2str(a.FoldNum)], 'store_all=0'});
            % Call the method "train" of the object "cv_model":
            cv_output   = train(cv_model, D); 
            % Collect the results
            OutX = []; OutY = []; ber =[];
            for kk = 1:a.FoldNum,
                outX    = cv_output.child{kk}.X;
                outY    = cv_output.child{kk}.Y;
                OutX    = [OutX; outX]; 
                OutY    = [OutY; outY]; 
                ber(kk) = balanced_errate(outX, outY);
            end
            % Check whether to do a bias correction
            if isa(my_model, 'chain')
                chain_length=length(my_model.child);
                if isa(my_model{chain_length}, 'bias')
                    correct_bias=1;
                end
            end
            if correct_bias
                % Train a bias model with the cv outputs
                [d, cv_bias]=train(bias, data(OutX, OutY));            
                % Compensate the bias
                OutX=OutX+cv_bias.b0;
            end
            % Compute the CV error rate and error bar
            cv_ber   = balanced_errate(OutX, OutY);
            cv_ebar    = std(ber,1)/sqrt(a.FoldNum);
            fprintf('CV BER (Inside PSMS) =%5.2f+-%5.2f%%\n', 100*cv_ber, 100*cv_ebar); 
            % Guessing the BER...
            guessed_ber=cv_ber; 
        end
        fitness=cv_ber;       % For correcting the BIAS          
        fprintf('\n-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-\n');
        
