function a = psms(algo,hyper) 

%=====================================================================================
%  Particle swarm model selection object      
%=====================================================================================   
%  a=psms(algo,hyperParam) Returns a psms object on algorithm algo using with 
%                        given Hyperparameters. 
% 
%  Possible hyperparameters (with defaults):
% SwarmSize=10;             - Number of particles, size of the swarm
% iterations=100;           - Number of iterations for searching
% Dim=25;                   - Dimensionality of each solution
% c1=2;                     - Weigth for the personal best solution 
% c2=2;                     - Weigth for the Global best solution
% w_start=0.95;             - Starting inertia weight
% w_end=0.4;                - Ending inertia weight
% w_varyfor=0.5;            - Vary for this fraction of iterations, that is
%                             if Iterations=100 and w_varyfor=0.5, the inertia 
%                             weight will be varied for 50 Iterations.                    
% Vmax=100;                 - Bound for the Max value of velocity
% TerminateErr=0;           - Error value that should be obtained in order
%                             to stop the search process
% TerminateIter=1;          - If 1, the algorithm will stop until
%                             Iterations have been performed
% a. folds=2;               - Number of folds for the cross validation
% k= a.save_every=1;        - Keep record of the best models every
%                             k-iterations.
% a.fms=1;                  - flag indicating full model selection (fms=1)
%                             or hyperparameter optimization and selection
%                             of preprocessing and feature selection
%                             methods (fms=0), or hyperparameter optimization of
%                             a classifier (fms=-1).
%
%  Model:
%   child                 - stored in child algorithm 
%
%  Methods:
%   train                 - selfexplanatory
%   test                  - selfexplanatory
%=====================================================================================
% Reference : Particle swarm model selection, In A. Saffari and I. Guyon editors, JMLR 
%             Special issue on model selection. (2008)
% Author    : Hugo Jair Escalante
% Link      : http://ccc.inaoep.mx/~hugojair
%=====================================================================================

% <<---- Initialisation of Hyperparams ---->>
% % %   Hyperparameters for the Swarm Optimization

a.SwarmSize=25;
a.Iterations=25;
a.Dim=25;
a.c1=2;
a.c2=2;
a.w_start=0.95;
a.w_end=0.4;
a.w_varyfor=0.5;
a.Vmax=100;
a.TerminateErr=0;
a.TerminateIter=1;
a.FoldNum=2;
a.Chi=1;
a.fms=1;
a.save_every=1;
a.create_ensemble=0;    

% <<---- Initialisation of model ---->>
  if nargin>0
   a.child{1}=algo;
  else
    % % %   Hyperparameters for the model selection process
    a.child{1}=zarbi; 
  end 

  if nargin==2,
    a.create_ensemble=hyper.create_ensemble;
    a.fms=hyper.fms;
    a.include_kridge=hyper.include_kridge;
    a.visualize=hyper.visualize;
    a.SwarmSize=hyper.swarm_size;
    a.Iterations=hyper.iterations;
    a.save_every=hyper.save_every;
  end;
% <<---- convert {} to group({}) ---->>
if isa(a.child{1},'cell') 
      a.child{1}=group(a.child{1}); 
end;

p=algorithm('psms');
a= class(a,'psms',p);

