% % % This function validates the values assigned to the particles. 
% % % According to the range of parameters for each of the considered
% % % methods.
% % % 
% % % Input: 
% % %           a           - A psms object
% % %           particle    - the vectorial representation of a model
% % %           n_features  - dimensionality of the data
% % % 
% % % Output:
% % %           valid_particle  - A validated particle 
% % % 
% % % The following particle representation-codification is used 
% % % [s2n, fmax, w_min, gs, fmax, relief, fmax, w_min, k_num, svcrfe, fmax, standarize, center, normalize, center, shift_n_scale,center,pc_extract,fmax,classifier,p1,p2,p3,p4,p5];
% % % 
% % % Value-Classifiers
% % 1- Zarbi
% % 2- Naive
% % 3- kridge
% % 4- Neural
% % 5- SVC
% % % 
% % %===============================================================
% % % Hugo Jair Escalante -- hugojair@ccc.inaoep.mx -- November 2007
% % %                        hugo.jair@gmail.com
% % %===============================================================

function [valid_particle] = validate_particle(a,particle, n_features)
 
binary_valued=[1,4,6,10,12,13,14,15,16,17,18];
nu_particle=zeros(1,length(particle));
uones=find(particle(binary_valued)>=0.5);
nu_particle(binary_valued(uones))=1;

% % % % Ensuring that only a single feature selection method is used
rango1=[4,6,10,18];rango2=[1,6,10,18];rango3=[1,4,10,18];rango4=[1,6,4,18];
rango5=[1,6,4,10];

% % % Update this 
if (nu_particle(1,1)==1)
    nu_particle(1,rango1)=0;
end
if (nu_particle(1,4)==1)
    nu_particle(1,rango2)=0;
end
if (nu_particle(1,6)==1)
    nu_particle(1,rango3)=0;
end
if (nu_particle(1,10)==1)
    nu_particle(1,rango4)=0;
end
if (nu_particle(1,18)==1)
    nu_particle(1,rango5)=0;
end

lb_feats=min(3,n_features);    %% lower bound on the number of features
ub_feats=n_features-1;
nn=10;                          %% parameters for relief's nns
lnn=1;
wmin_max=0.5;%%1
wmin_min=0.1;
fmax_valued=[2,5,7,11,19];
nu_particle(1,fmax_valued)=round(particle(1,fmax_valued));
for i=1:length(fmax_valued)
    if (nu_particle(fmax_valued(i)) > ub_feats )
        nu_particle(fmax_valued(i))=ub_feats;
    elseif (nu_particle(fmax_valued(i)) < lb_feats )
        nu_particle(fmax_valued(i))=lb_feats;
    end
end
nu_particle(9)=round(particle(9));
if (particle(9) > nn )
        nu_particle(9)=nn;
end
if (particle(9) < 2)
        nu_particle(9)=lnn;
end
wmin_valued=[3,8];
nu_particle(1,wmin_valued)=particle(1,wmin_valued);
for i=1:length(wmin_valued)
    if (nu_particle(wmin_valued(i)) > wmin_max )
        nu_particle(wmin_valued(i))=wmin_max;
    elseif (nu_particle(wmin_valued(i)) < wmin_min )
        nu_particle(wmin_valued(i))=wmin_min;
    end
end

% % % Classifier
if (a.fms~=1)  %%%% If the task is just hyperparameter optimization of the selected model
[classifier,cval]=get_classifier_clop(a,a.child{1}.child{1,1}.name);
nu_particle(20)=cval;
else         %%%% If we should perform full model selection
nu_particle(20)=particle(20);
classifier=get_classifier(a,nu_particle(20));
end 

% % % If the classifier is not Zarbi nor Naive Bayes we can check its
% hyperparameters
if (~(classifier==1 || classifier==2))
% % % If the classifier is a Neural Net
[classifier classifierlb classifierub]=get_classifier_pars(a,classifier);
    if ((classifier==4)) 

% % Units
        if (round(particle(21)) >= classifierlb(1,1) && round(particle(21)) <= classifierub(1,1))
            nu_particle(21)=round(particle(21));
        else
            nu_particle(21)=classifierub(1,1);
        end
% % Shrinkage      
        if (particle(22) <= classifierlb(1,2))
            nu_particle(22)=classifierlb(1,2);
        elseif (particle(22)>classifierub(1,2))
            nu_particle(22)=abs(classifierlb(1,2)-rand);
        else
            nu_particle(22)=particle(22);
        end
% %     Balance
        if (particle(23)>0.5)
            nu_particle(23)=1;
        else nu_particle(23)=0;
        end
% % Max - iterations    
        if (round(particle(24)) >= classifierlb(1,4) && round(particle(24)) <= classifierub(1,4) )
            nu_particle(24)=round(particle(24));
        else
            nu_particle(24)=classifierub(1,4);
        end
    
% % % or If the classifier is a kernel method
elseif (classifier==3 || classifier==5)
%         max_shrink=2;
% % Coef0    
        if (particle(21)>0.5)
            nu_particle(21)=1;
        else
            nu_particle(21)=0;
        end
% %   Degree   & Gamma
        if (particle(25)>0.5)
            if (round(particle(22)) < classifierlb(1,2)) 
                    nu_particle(22)=classifierlb(1,2);
            elseif (particle(22)>classifierub(1,2)) 
                particle(22)=classifierub(1,2);
            else
                nu_particle(22)=round(particle(22));    
            end
            nu_particle(23)=0;
        else
            nu_particle(22)=0;    
            if (round(particle(23)) <= classifierlb(1,3) || round(particle(23)) >= classifierub(1,3)) 
                    nu_particle(23)=rand;
            else
                nu_particle(23)=particle(23);
            end        
        end
% %     Shrinkage
        if (particle(24) <= classifierlb(1,4))
            nu_particle(24)=rand;
        elseif (particle(24)>classifierub(1,4))
            nu_particle(24)=abs(classifierub(1,4)-rand);
        else
            nu_particle(24)=particle(24);
        end

end
end 

valid_particle=nu_particle;

